<?php
/**
 * @copyright  Copyright (c) 2018 Icredit 
 * @version 1.6.1
 * @author iCredit
*/

if (!defined('_PS_VERSION_'))
	exit;

class ICredit extends PaymentModule
{
	# Custom define variable
	# Next line
	const NEXT_LINE = "\n";
	
	# Require mark
	const REQUIRE_MARK = '<span style="color: #F00;">*</span>&nbsp;';
	
	# POST error
	private $post_error = array();
	
	# HTML content
	private $html_cont = '';
	
	# Order status list
	private $order_status_list = array();
	
	# Allpay parameters
	private $allpay_param = array(
		'TestMode'
		, 'PaymentMethodDescription'
		, 'Title'
		, 'TestGroupToken'
		, 'ProdGroupToken'
		, 'SecondProdGroupToken'
		, 'WindowType'
		, 'iFramePopupHeight'
		, 'iFramePopupWidth'
		, 'HideItemsList'
		, 'InvoiceLanguage'
		, 'ExemptVAT'
		, 'RedirectURL'
		, 'IPNURL'
		, 'OrderCreateStatus'
		, 'PaidStatus'
		, 'UnpaidStatus'
	);


	
	public function __construct()
	{
		# The value MUST be the name of the module's folder.
		$this->name = 'iCredit';
		
		# The type of the module, displayed in the modules list.
		$this->tab = 'payments_gateways';
		
		# The version number for the module, displayed in the modules list.
		$this->version = '1.6.1';
		
		# The auther for the module, displayed in the modules list.
		$this->author = 'iCredit';
		$this->bootstrap = 1;
		
		# Calling the parent constuctor method must be done after the creation of the $this->name variable and before any use of the $this->l() translation method.
		parent::__construct();
		
		# A name for the module, which will be displayed in the back-office's modules list.
		$this->displayName = $this->l('iCredit payment');
		
		# A description for the module, which will be displayed in the back-office's modules list.
		$this->description = $this->l('iCredit payment gateway');
		
		# A message, asking the administrator if he really does want to uninstall the module. To be used in the installation code.
		$this->confirmUninstall = $this->l('Are you sure you want to uninstall iCredit payment module?');
		
		# Custom variable: Get the order status
		$this->order_status_list = $this->_getOrderStatus();
		
	}
	
	# Perform checks and actions during the module's installation process
	public function install()
	{
		# ?? PrestaShop 內? Hook 進?註? payment ? paymentReturn
		if (!parent::install() OR !$this->_hookAllpayPaymentPage())
		{
			return false;
		}
		else
		{
			return true;
		}
	}
		
	public function uninstall()
	{
		if (!parent::uninstall() or !$this->_cleanAllpayConfig())
		{
			return false;
		}
		else
		{
			return true;
		}
	}
		
	public function getContent()
	{
		$html_cont = '';
		if (Tools::isSubmit($this->name . '_submit'))
		{
			# Check the POST parameters
			$this->_postValidation();
			
			if (!empty($this->post_error))
			{
				# Display the POST error
				foreach ($this->post_error as $tmp_err)
				{
					$html_cont .= $this->_getErrorMessage($tmp_err);
				}
			}
			else
			{
				$html_cont .= $this->_postProcess();
			}
		}
		$html_cont .= $this->_displayForm();
		
		return $html_cont;
	}
	
	public function hookPayment($params)
	{
		
		if (!$this->active)
		{
			return;
		}
		if (!$this->checkCurrency($params['cart']))
		{
			return;
		}
		
		$this->smarty->assign(
			array(
				'allpay_img_path' => $this->_path . 'img/',
				'description'=> Configuration::get('PaymentMethodDescription'),
				'title'=> Configuration::get('Title'),
			)
		);
		
		return $this->display(__FILE__, 'payment.tpl');
	}

	public function hookPaymentReturn($params)
	{
		if (!$this->active)
			return;

		$state = $params['objOrder']->getCurrentState();
		if (in_array($state, array(Configuration::get('OrderCreateStatus'), Configuration::get('PaidStatus'), Configuration::get('UnpaidStatus'))))
		{
			$this->smarty->assign(array(
				'total_to_pay' => Tools::displayPrice($params['total_to_pay'], $params['currencyObj'], false),				
				'status' => 'ok',
				'id_order' => $params['objOrder']->id,
				'status' => $params['status']
			));
			if (isset($params['objOrder']->reference) && !empty($params['objOrder']->reference))
				$this->smarty->assign('reference', $params['objOrder']->reference);
		}
		else
			$this->smarty->assign('status', 'failed');
		return $this->display(__FILE__, 'payment_return.tpl');
	}
	
	public function checkCurrency($cart)
	{
		$currency_order = new Currency($cart->id_currency);
		$currencies_module = $this->getCurrency($cart->id_currency);

		if (is_array($currencies_module))
		{
			foreach ($currencies_module as $currency_module)
			{
				if ($currency_order->id == $currency_module['id_currency'])
				{
					return true;
				}
			}
		}
		//echo "<pre>";print_r($currencies_module);die;
			
		return false;
	}
	
	# Public custom function
	public function _splitByCommon($source_str)
	{
		$splited_array = array();
		if (!empty($source_str))
		{
			$splited_array = explode(',', $source_str);
		}
		
		return $splited_array;
	}
	
	public function _getPaymentDescription($payment_name)
	{
		// if (array_key_exists($payment_name, $this->payments_description))
		// {
		// 	return $this->payments_description[$payment_name];
		// }
		// else
		// {
		// 	return $this->l('Undefine payment');
		// }
	}
	
	# Private custom function
	private function _postValidation()
	{
		// $require_fields = array('Merchant ID', 'Hash Key', 'Hash IV');
		
		// foreach ($require_fields as $tmp_field)
		// {
		// 	if (!(Tools::getValue(str_replace(' ', '', $tmp_field))))
		// 	{
		// 		array_push($this->post_error, $this->l($tmp_field . ' is required'));
		// 		break;
		// 	}
		// }
	}
	
	private function _getSelect($select_name, $options, $default_value = '')
	{
		$html_select = $this->_setHTML('          <select name="' . $select_name . '" class="fixed-width-xl">');
		foreach ($options as $tmp_row)
		{
			$selected = ($tmp_row['value'] == $default_value ? ' selected' : '');
			$html_select .= $this->_setHTML('            <option value="' . $tmp_row['value'] . '"' . $selected . '>' . $tmp_row['text'] . '</option>');
		}
		$html_select .= '          </select>';
		
		return $html_select;
	}
	
	private function _getInputOption($description, $input_option, $not_null = true)
	{
		$html_input = $this->_setHTML('     <div class="form-group">');
		$html_input .= $this->_setHTML('        <label class="control-label col-lg-3">');
		$html_input .= $this->_setHTML(($not_null ? self::REQUIRE_MARK : '') . $description);
		$html_input .= $this->_setHTML('        </label>');
		$html_input .= $this->_setHTML('        <div class="col-lg-9 ">');
		$html_input .= $this->_setHTML($input_option);
		$html_input .= $this->_setHTML('        </div>');
		$html_input .= $this->_setHTML('      </div>');
		
		return $html_input;
	}
	
	private function _setHTML($html_content)
	{
		return $html_content . self::NEXT_LINE;
	}
	
	private function _getOptionValue($var_name, $default_value)
	{
		$config_value = Configuration::get($var_name);
		if (!$config_value)
		{
			return $default_value;
		}
		else
		{
			return $config_value;
		}
	}
	
	private function _displayForm()
	{
		$html_form = $this->_setHTML('<form action="' . Tools::htmlentitiesUTF8($_SERVER['REQUEST_URI']) . '" method="post" class="defaultForm form-horizontal">');
		$html_form .= $this->_setHTML('<div id="fieldset_0" class="panel">');
		$html_form .= $this->_setHTML('   <div class="panel-heading"><img src="../img/admin/contact.gif" />' . $this->l('iCredit configuration') . '</div>');
		$html_form .= $this->_setHTML('  <div class="form-wrapper">');
		
		# Test Mode
		$select_option = array(
			array('text' => $this->l('No'), 'value' => '0')
			, array('text' => $this->l('Yes'), 'value' => '1')
		);
		
		$html_select = $this->_getSelect('TestMode', $select_option, $this->_getOptionValue('TestMode', '')) ;
		$html_form .= $this->_getInputOption($this->l('Test Mode'), $html_select, false);

		# Payment Method Description
		$html_text = '          <textarea name="PaymentMethodDescription" value="' . $this->_getOptionValue('PaymentMethodDescription', '') . '" style="width: 300px;" >'.$this->_getOptionValue('PaymentMethodDescription', '').'</textarea>';
		$html_form .= $this->_getInputOption($this->l('Payment Method Description'), $html_text, false);

		# Payment Title
		$html_text = '          <input type="text" name="Title" value="' . $this->_getOptionValue('Title', '') . '" style="width: 300px;" />';
		$html_form .= $this->_getInputOption($this->l('Title'), $html_text, false);


		# Test group token
		$html_text = '          <input type="text" name="TestGroupToken" value="' . $this->_getOptionValue('TestGroupToken', '') . '" style="width: 300px;" />';
		$html_form .= $this->_getInputOption($this->l('Test Group Token'), $html_text, false);

		# Prod group token
		$html_text = '          <input type="text" name="ProdGroupToken" value="' . $this->_getOptionValue('ProdGroupToken', '') . '" style="width: 300px;" />';
		$html_form .= $this->_getInputOption($this->l('Prod Group Token'), $html_text, false);
                
                #Second group token
                $html_text = '          <input type="text" name="SecondProdGroupToken" value="' . $this->_getOptionValue('SecondProdGroupToken', '') . '" style="width: 300px;" />';
		$html_form .= $this->_getInputOption($this->l(' Second Prod Group Token'), $html_text, false);

		# Max Payment 
		/*$html_text = '          <input type="text" name="MaxPayment" value="' . $this->_getOptionValue('MaxPayment', '') . '" style="width: 300px;" />';
		$html_form .= $this->_getInputOption($this->l('Max Payment'), $html_text, false);*/

		# Credit From Payment 
		/*$html_text = '          <input type="text" name="CreditFromPayment" value="' . $this->_getOptionValue('CreditFromPayment', '') . '" style="width: 300px;" />';
		$html_form .= $this->_getInputOption($this->l('Credit From Payment'), $html_text, false);*/

		# Window Type
		$select_window_option = array(
			array('text' => $this->l('Redirect'), 'value' => 'redirect')
			, array('text' => $this->l('Popup'), 'value' => 'popup')
			, array('text' => $this->l('iFrame'), 'value' => 'iframe')
		);
		
		$html_select = $this->_getSelect('WindowType', $select_window_option, $this->_getOptionValue('WindowType', '')) ;
		$html_form .= $this->_getInputOption($this->l('Window Type'), $html_select, false);

        # iFrame/Popup Height 
		$html_text = '          <input type="text" name="iFramePopupHeight" value="' . $this->_getOptionValue('iFramePopupHeight', '') . '" style="width: 300px;" />';
		$html_form .= $this->_getInputOption($this->l('iFrame/Popup Height'), $html_text, false);

	    # iFrame/Popup Width 
		$html_text = '          <input type="text" name="iFramePopupWidth" value="' . $this->_getOptionValue('iFramePopupWidth', '') . '" style="width: 300px;" />';
		$html_form .= $this->_getInputOption($this->l('iFrame/Popup Width'), $html_text, false);


		# Hide Item List
		$select_item_option = array(
			array('text' => $this->l('Yes'), 'value' => '0')
			, array('text' => $this->l('No'), 'value' => '1')
			
		);		
		$html_select = $this->_getSelect('HideItemsList', $select_item_option, $this->_getOptionValue('HideItemsList', '')) ;
		$html_form .= $this->_getInputOption($this->l('Hide Item List'), $html_select, false);

		# Create Token
		/*$select_item_option = array(
			array('text' => $this->l('Yes'), 'value' => '0')
			, array('text' => $this->l('No'), 'value' => '1')
			
		);		
		$html_select = $this->_getSelect('CreateToken', $select_item_option, $this->_getOptionValue('CreateToken', '')) ;
		$html_form .= $this->_getInputOption($this->l('Create Token'), $html_select, false);*/

		# Create Token
		// $available_payments = $this->_splitByCommon(Configuration::get('CreateToken'));
		// $html_checkbox = '';
		// foreach ($this->allpay_payments as $payment_name)
		// {
		// 	$is_checked = ((!empty($available_payments) and in_array($payment_name, $available_payments)) ? ' checked' : '');
		// 	$html_checkbox .= $this->_setHTML('&nbsp;<input type="radio" name="' . $payment_name . '"' . $is_checked . ' value="1" />&nbsp;' . $this->_getPaymentDescription($payment_name) . '<br />');
		// }
		// $html_form .= $this->_getInputOption($this->l('Available Payments'), $html_checkbox, false);

        # Invoice Language
		$select_lang_option = array(
			array('text' => $this->l('Always English'), 'value' => 'english')
			, array('text' => $this->l('Always Hebrew'), 'value' => 'hebrew')
			, array('text' => $this->l('By Billing Address'), 'value' => 'billing')
			, array('text' => $this->l('By Shipping Address'), 'value' => 'shipping')
			, array('text' => $this->l('By Shipping or Billing Address'), 'value' => 'billingorshipping')
		);
		
		$html_select = $this->_getSelect('InvoiceLanguage', $select_lang_option, $this->_getOptionValue('InvoiceLanguage', '')) ;
		$html_form .= $this->_getInputOption($this->l('Invoice Language'), $html_select, false);


		# Exempt VAT
		$select_vat_option = array(
			array('text' => $this->l('Always Charge VAT'), 'value' => 'chargevat')
			, array('text' => $this->l('Always Exempt'), 'value' => 'exempt')
			, array('text' => $this->l('By Billing Address'), 'value' => 'billing')
			, array('text' => $this->l('By Shipping Address'), 'value' => 'shipping')
			, array('text' => $this->l('By Shipping or Billing Address'), 'value' => 'billingorshipping')
		);
		
		$html_select = $this->_getSelect('ExemptVAT', $select_vat_option, $this->_getOptionValue('ExemptVAT', '')) ;
		$html_form .= $this->_getInputOption($this->l('Exempt VAT'), $html_select, false);

		
		# Redirect URL 
		$html_text = '          <input type="text" name="RedirectURL" value="' . $this->_getOptionValue('RedirectURL', '') . '" style="width: 300px;" />';
		$html_form .= $this->_getInputOption($this->l('Redirect URL'), $html_text, false);


		# IPN URL
		$html_text = '          <input type="text" name="IPNURL" value="' . $this->_getOptionValue('IPNURL', '') . '" style="width: 300px;" />';
		$html_form .= $this->_getInputOption($this->l('IPN URL'), $html_text, false);


		
		# Order Create Status
		$html_select = $this->_getSelect('OrderCreateStatus', $this->order_status_list, $this->_getOptionValue('OrderCreateStatus', ''));
		$html_form .= $this->_getInputOption($this->l('Order Create Status'), $html_select, false);
		
		# Paid Status
		$html_select = $this->_getSelect('PaidStatus', $this->order_status_list, $this->_getOptionValue('PaidStatus', ''));
		$html_form .= $this->_getInputOption($this->l('Paid Status'), $html_select, false);
		
		# Unpaid/Paid Failed Status
		$html_select = $this->_getSelect('UnpaidStatus', $this->order_status_list, $this->_getOptionValue('UnpaidStatus', ''));
		$html_form .= $this->_getInputOption($this->l('Unpaid/Paid Failed Status'), $html_select, false);
		

		
		

		$html_form .= '      </div>>';
		$html_form .= '        <div class="panel-footer">>';
		$html_form .= '         <button class="btn btn-default pull-right" name="' . $this->name . '_submit" id="icredit_form_submit_btn" value="1" type="submit">
							<i class="process-icon-save"></i> Save
						</button>';		
		$html_form .= '        </div>';
		$html_form .= '    </div>';
		$html_form .= '  </fieldset>';
		$html_form .= '</form>';
		

		return $html_form;
	}
																																		
	
	private function _postProcess()
	{
		# Update Allpay parameters
		//echo "<pre>"; print_r($this->allpay_param); die;
		foreach ($this->allpay_param as $param_name)
		{
			
			if (!Configuration::updateValue($param_name, Tools::getValue($param_name)))
			{
				
				return $this->_getErrorMessage($param_name . ' ' . $this->l('updated failed'));
			}
			
		}
	
			
		return $this->_getSuccessMessage($this->l('Settings updated'));
		
	}
	
	private function _getErrorMessage($message)
	{
		return '<div class="alert error">' . $message . '</div>';
	}
	
	private function _getSuccessMessage($message)
	{
		return '<div class="module_confirmation conf confirm alert alert-success"> ' . $message . '</div>';
	}

	private function _cleanAllpayConfig()
	{
		$allpay_config = $this->allpay_param;
		array_push($allpay_config, 'AvailablePayments');
		array_push($allpay_config, 'CreditInstallments');
		foreach ($allpay_config as $param_name)
		{
			if (!Configuration::deleteByName($param_name))
			{
				return false;
			}
		}
		//echo "<pre>";print_r($allpay_config);die;
		
		return true;
	}

	private function _hookAllpayPaymentPage()
	{
		// $hook_list = array('payment');
		$hook_list = array('payment', 'paymentReturn');
		foreach ($hook_list as $hook_name)
		{
			if (!$this->registerHook($hook_name))
			{
				return false;
			}
		}
		//echo "<pre>";print_r($hook_list);die;
		return true;
	}
	
	private function _getOrderStatus()
	{
		$select_sql = 'SELECT a.`id_order_state`, a.`module_name`, b.`name`';
		$select_sql .= ' FROM `' . _DB_PREFIX_ . 'order_state` a LEFT JOIN `' . _DB_PREFIX_ . 'order_state_lang` b';
		$select_sql .= ' ON (b.`id_order_state` = a.`id_order_state` AND b.`id_lang` = ' . (int)$this->context->language->id . ')';
		$select_sql .= ' ORDER BY a.`id_order_state` ASC LIMIT 0,50;';
		$tmp_list = Db::getInstance()->executeS($select_sql);
		$status_list = array();
		foreach ($tmp_list as $list_row)
		{
			if (empty($list_row['module_name']) or $list_row['module_name'] == 'iCredit')
			{
				$tmp_row = array('value' => $list_row['id_order_state'], 'text' => $list_row['name']);
				array_push($status_list, $tmp_row);
			}
		}

		//echo "<pre>";print_r($status_list);die;
		return $status_list;
	}

	public static function getShopDomainSsl($http = false, $entities = false)
	{
		if (method_exists('Tools', 'getShopDomainSsl'))
			return Tools::getShopDomainSsl($http, $entities);
		else
		{
			if (!($domain = Configuration::get('PS_SHOP_DOMAIN_SSL')))
				$domain = self::getHttpHost();
			if ($entities)
				$domain = htmlspecialchars($domain, ENT_COMPAT, 'UTF-8');
			if ($http)
				$domain = (Configuration::get('PS_SSL_ENABLED') ? 'https://' : 'http://').$domain;
			return $domain;
		}
	}

	
}
